package com.guo.aly;

import com.alibaba.fastjson2.JSON;
import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.Common;
import com.aliyun.teautil.models.RuntimeOptions;
import com.guo.aly.config.AliConfig;
import com.guo.aly.model.EcsSelectModel;
import com.guo.aly.model.EscRemoveModel;
import com.guo.aly.model.InstanceInfo;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author gxb
 * @description TODO
 * @date 2024-04-18 16:15
 */
@Service
@Log4j2
public class ALYunEcsService {

    private final AliConfig aliConfig;

    private final Client client;

    public ALYunEcsService(AliConfig aliConfig, Client client) {
        this.aliConfig = aliConfig;
        this.client = client;
    }

    /**
     * 根据实例ID和实例名称查询实例信息
     * @param ecsSelectModel
     * @return  返回实例集合信息
     */
    public List<InstanceInfo> selectECS(EcsSelectModel ecsSelectModel) throws Exception {
        DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest()
                .setRegionId(aliConfig.getRegionId())
                .setPageSize(10);

        // 判断是否指定了实例ID或实例名称
        if (ecsSelectModel.getInstanceIdList() != null && !ecsSelectModel.getInstanceIdList().isEmpty()) {
            // 如果指定了实例ID，则设置实例ID查询条件
            describeInstancesRequest.setInstanceIds(String.join(",", ecsSelectModel.getInstanceIdList()));
        } else if (ecsSelectModel.getInstanceNameList() != null && !ecsSelectModel.getInstanceNameList().isEmpty()) {
            // 如果指定了实例名称，则设置实例名称查询条件
            describeInstancesRequest.setInstanceName(String.join(",", ecsSelectModel.getInstanceNameList()));
        } else {
            // 如果既没有指定实例ID也没有指定实例名称，则抛出异常或者返回空列表，视情况而定
            throw new IllegalArgumentException("Please provide at least one instance ID or instance name.");
            // 或者直接返回空列表
            // return new ArrayList<>();
        }

        List<InstanceInfo> instanceInfos = new ArrayList<>(); // 用于存储查询到的实例信息

        try {
            com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
            DescribeInstancesResponse describeInstancesResponse = client.describeInstancesWithOptions(describeInstancesRequest, runtime);
            DescribeInstancesResponseBody body = describeInstancesResponse.getBody();
            DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstances instances = body.getInstances();
            List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> instanceList = instances.getInstance();

            // 修改 selectECS 方法中 IP 地址的处理部分
            for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance item : instanceList) {
                InstanceInfo instanceInfo = new InstanceInfo();
                instanceInfo.setInstanceId(item.getInstanceId());
                instanceInfo.setInstanceName(item.getInstanceName());
                String publicIpAddress = item.getPublicIpAddress().getIpAddress().toString();
                // 去掉方括号
                publicIpAddress = publicIpAddress.substring(1, publicIpAddress.length() - 1);
                instanceInfo.setPublicIpAddress(publicIpAddress);
                String privateIpAddress = item.getVpcAttributes().getPrivateIpAddress().getIpAddress().toString();
                // 去掉方括号
                privateIpAddress = privateIpAddress.substring(1, privateIpAddress.length() - 1);
                instanceInfo.setPrivateIpAddress(privateIpAddress);
                instanceInfos.add(instanceInfo);
            }
        } catch (TeaException error) {
            error.printStackTrace();
            // 可以打印错误信息以便排查问题
            System.out.println("TeaException occurred: " + error.getMessage());
            // 异常处理
        } catch (Exception _error) {
            _error.printStackTrace();
            // 可以打印错误信息以便排查问题
            System.out.println("Exception occurred: " + _error.getMessage());
            // 异常处理
        }

        return instanceInfos;
    }




    /**
     * 创建实例方法
     * @throws Exception
     */
    public  String createAnServer(String isNull) throws Exception {
        // 地域Id
        //String regionId = "cn-shanghai";
        String regionId = aliConfig.getRegionId();
        // 镜像 ID，启动实例时选择的镜像资源。
        // String imageId = "m-uf6elrscl3c9wk6o762l";
        String imageId = aliConfig.getImageId();
        // 实例规格
        //String instanceType = "ecs.u1-c1m1.large";
        String instanceType = aliConfig.getInstanceType();
        // 新创建实例所属于的安全组 ID。
        //String securityGroupId = "sg-uf6bj6vxp8ruhvffdsau";
        String securityGroupId = aliConfig.getSecurityGroupId();
        // 虚拟交换机 ID。
        //String vSwitchId = "vsw-uf66jtgij0ptqxf1ix6l7   ";
        String vSwitchId = aliConfig.getVSwitchId();
        // 公网出带宽最大值，单位为 Mbit/s。取值范围：0~100。 默认值：0。
        //Integer internetMaxBandwidthOut = Integer.parseInt("2");
        Integer internetMaxBandwidthOut = Integer.parseInt(aliConfig.getInternetMaxBandwidthOut());
        // 网络计费类型。取值范围：
        // PayByBandwidth: 按固定带宽计费。
        // PayByTraffic: 按使用流量计费。
        // 默认值：PayByTraffic。
        //String internetChargeType = "PayByTraffic";
        String internetChargeType = aliConfig.getInternetChargeType();
        // 系统盘大小
        //String size = "20";
        String size = aliConfig.getSize();
        // 系统盘的云盘种类
        //String category = "cloud_essd";
        String category = aliConfig.getCategory();
        // ECS实例的计费方式
        // PrePaid：包年包月
        // PostPaid：按量付费
        //String instanceChargeType = "PostPaid";
        String instanceChargeType = aliConfig.getInstanceChargeType();
        // 创建 【1台】 实例
        if (isNull == null){
            String instances = RunInstance(client, regionId, imageId, instanceType, securityGroupId, vSwitchId, internetMaxBandwidthOut, internetChargeType, size, category, instanceChargeType);
            return instances; //返回实例ID
        }
        // 批量创建实例
        String instances = RunInstances(client, regionId, imageId, instanceType, securityGroupId, vSwitchId, internetMaxBandwidthOut, internetChargeType, size, category, instanceChargeType);
        //返回实例ID
        return instances;
    }


    /**
     * 批量创建【2】台服务器
     * RunInstances     通过备选实例规格创建ECS实例最佳实践
     * 该场景中，在调用RunInstances创建ECS实例时判断是否发生库存不足等错误，如果发生错误，将调用DescribeRecommendInstanceType查询备选实例，然后通过备选实例规格重新创建ECS实例。
     */
    public  String RunInstances(Client client, String regionId, String imageId, String instanceType, String securityGroupId, String vSwitchId, Integer internetMaxBandwidthOut, String internetChargeType, String size, String category, String instanceChargeType) throws Exception {
        System.setOut(new java.io.PrintStream(System.out, true, "UTF-8"));
        RunInstancesRequest request1 = new RunInstancesRequest()
                .setRegionId(regionId)
                .setImageId(imageId)
                .setInstanceType(instanceType)
                .setSecurityGroupId(securityGroupId)
                .setVSwitchId(vSwitchId)
                .setInstanceName("Myname")
                .setDescription("Myprocure")
                .setInternetMaxBandwidthOut(internetMaxBandwidthOut)
                .setInternetChargeType(internetChargeType)
                .setInstanceChargeType(instanceChargeType)
                // 批量创建五台ECS实例，如果不设置该参数，默认创建一台ECS实例。
                .setAmount(2)
                // 如果缺少库存可以接受的最低创建数量。
                // minAmount = 2,
                // 打开预检参数功能，不会实际创建ECS实例，只检查参数正确性、用户权限或者ECS库存等问题。
                // 实际情况下，设置了DryRun参数后，Amount必须为1，MinAmount必须为空，您可以根据实际需求修改代码。
                .setDryRun(false)
                .setSystemDisk(new RunInstancesRequest.RunInstancesRequestSystemDisk()
                        .setSize(size)
                        .setCategory(category));
        RunInstancesResponse responces = client.runInstances(request1);
        try {
            log.info("--------------------批量创建实例开始--------------------");

            log.info("--------------------创建实例成功，实例ID:" + JSON.toJSONString(responces.body.instanceIdSets.instanceIdSet) + "--------------------");
        } catch (TeaException error) {
            log.error("--------------------创建实例失败:" + error+ "--------------------");
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            log.error("--------------------创建实例失败:" +error + "--------------------");
        }
        return  JSON.toJSONString(responces.body.instanceIdSets.instanceIdSet);
    }


    /**
     * 批量创建【1】台服务器
     * RunInstances     通过备选实例规格创建ECS实例最佳实践
     * 该场景中，在调用RunInstances创建ECS实例时判断是否发生库存不足等错误，如果发生错误，将调用DescribeRecommendInstanceType查询备选实例，然后通过备选实例规格重新创建ECS实例。
     */
    public  String RunInstance(Client client, String regionId, String imageId, String instanceType, String securityGroupId, String vSwitchId, Integer internetMaxBandwidthOut, String internetChargeType, String size, String category, String instanceChargeType) throws Exception {
        System.setOut(new java.io.PrintStream(System.out, true, "UTF-8"));
        RunInstancesRequest request1 = new RunInstancesRequest()
                .setRegionId(regionId)
                .setImageId(imageId)
                .setInstanceType(instanceType)
                .setSecurityGroupId(securityGroupId)
                .setVSwitchId(vSwitchId)
                .setInstanceName("Myname")
                .setDescription("Myprocure")
                .setInternetMaxBandwidthOut(internetMaxBandwidthOut)
                .setInternetChargeType(internetChargeType)
                .setInstanceChargeType(instanceChargeType)
                // 批量创建五台ECS实例，如果不设置该参数，默认创建一台ECS实例。
                //.setAmount(2)
                // 如果缺少库存可以接受的最低创建数量。
                // minAmount = 2,
                // 打开预检参数功能，不会实际创建ECS实例，只检查参数正确性、用户权限或者ECS库存等问题。
                // 实际情况下，设置了DryRun参数后，Amount必须为1，MinAmount必须为空，您可以根据实际需求修改代码。
                .setDryRun(false)
                .setSystemDisk(new RunInstancesRequest.RunInstancesRequestSystemDisk()
                        .setSize(size)
                        .setCategory(category));
        RunInstancesResponse responces = client.runInstances(request1);
        try {
            log.info("--------------------批量创建实例开始--------------------");

            log.info("--------------------创建实例成功，实例ID:" + JSON.toJSONString(responces.body.instanceIdSets.instanceIdSet) + "--------------------");
        } catch (TeaException error) {
            log.error("--------------------创建实例失败:" + error+ "--------------------");
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            log.error("--------------------创建实例失败:" +error + "--------------------");
        }
        return  JSON.toJSONString(responces.body.instanceIdSets.instanceIdSet);
    }


    public  DescribeInstancesResponse DescribeInstances(Client client, String regionId, String instanceIds, String instanceName) throws Exception {
        DescribeInstancesRequest req = new DescribeInstancesRequest()
                .setRegionId(regionId)
                .setInstanceName(instanceName);
        if (!com.aliyun.teautil.Common.empty(instanceIds)) {
            req.instanceIds = JSON.toJSONString(com.aliyun.darabonbastring.Client.split(instanceIds, ",", 50));
        }

        DescribeInstancesResponse resp = client.describeInstances(req);
        log.error("--------------------查询需要删除的实例--------------------");
        return resp;
    }

    public  void ModifyInstanceAttribute(Client client, String instatnceId) throws Exception {
        ModifyInstanceAttributeRequest req = new ModifyInstanceAttributeRequest()
                .setInstanceId(instatnceId)
                .setDeletionProtection(false);
        client.modifyInstanceAttribute(req);
        log.info("--------------------" + instatnceId + "释放保护取消成功--------------------");
    }

    public  void DeleteInstances(Client client, String regionId, String instanceIds, String force) throws Exception {
        DeleteInstancesRequest req = new DeleteInstancesRequest()
                .setRegionId(regionId)
                .setInstanceId(com.aliyun.darabonbastring.Client.split(instanceIds, ",", 50))
                .setForce(com.aliyun.teautil.Common.equalString(force, "true"));
        DeleteInstancesResponse resp = client.deleteInstances(req);
        log.info("--------------------实例释放成功--------------------");
        log.info(JSON.toJSONString(com.aliyun.teautil.Common.toMap(resp)));
    }

    /**
     * 批量删除实力
     * @param
     * @throws Exception
     */
    public  void DeleServerCreateAn(EscRemoveModel escRemoveModel) throws Exception {
        // 区域ID
        //String regionId = "cn-shanghai";
        String regionId = aliConfig.getRegionId();
        // 多个实例ID，用英文逗号分隔
        //String instanceIds = "i-uf6h4s0jtpvobykd7vzc";
        String instanceIds = escRemoveModel.getInstanceIds();
        // 实例名称，支持使用通配符*进行模糊搜索
        //String instanceName = "MyFirstEcsInstance";
        String instanceName = escRemoveModel.getInstanceName();
        // 强制删除有删除保护的机器
        //String deleteProtected = "true";
        String deleteProtected = "true";
        // 强制删除运行中的机器
        //String force = "true";
        String force = "true";
        if (com.aliyun.teautil.Common.equalString(deleteProtected, force)) {
            DescribeInstancesResponse describeInstancesResp = DescribeInstances(client, regionId, instanceIds, instanceName);
            instanceIds = "";
            for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : describeInstancesResp.body.instances.instance) {
                instanceIds = "" + instance.instanceId + "," + instanceIds + "";
                if (instance.deletionProtection) {
                    ModifyInstanceAttribute(client, instance.instanceId);
                }

            }
            instanceIds = com.aliyun.darabonbastring.Client.subString(instanceIds, 0, -1);
        }

        if (com.aliyun.teautil.Common.empty(instanceIds)) {
            log.info("--------------------无有效实例可删除--------------------");
            return ;
        }

        DeleteInstances(client, regionId, instanceIds, force);
    }




}
