package com.guo.aly.config;

import com.aliyun.ecs20140526.Client;
import com.aliyun.teaopenapi.models.Config;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author gxb
 * @description TODO
 * @date 2024-04-18 16:16
 */
@Data
@Configuration
@ConfigurationProperties(prefix = "aliyun")
public class AliConfig {

    private String accessKeyId;

    private String accessKeySecret;

    /**
     * 地域ID
     */
    private String regionId;

    /**
     * 镜像ID
     */
    private String imageId;

    /**
     * 实例规格
     */
    private String instanceType;

    /**
     * 安全组ID
     */
    private String securityGroupId;

    /**
     * 虚拟交换机ID
     */
    private String vSwitchId;

    /**
     * 公网出带宽最大值，单位为 Mbit/s。取值范围：0~100。 默认值：0。
     */
    private String internetMaxBandwidthOut;

    /**
     * 网络计费类型，取值范围
     */
    private String internetChargeType;

    /**
     * 系统盘大小
     */
    private String size;

    /**
     * 系统盘的云盘种类
     */
    private String category;

    /**
     * ECS实例的计费方式
     */
    private String instanceChargeType;

    @Bean
    public  Client createClient(AliConfig aliConfig) throws Exception {
        // 工程代码泄露可能会导致 AccessKey 泄露，并威胁账号下所有资源的安全性。以下代码示例仅供参考。
        // 建议使用更安全的 STS 方式，更多鉴权访问方式请参见：https://help.aliyun.com/document_detail/378657.html。
        Config config = new Config()
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_ID。
                .setAccessKeyId(aliConfig.getAccessKeyId())
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_SECRET。
                .setAccessKeySecret(aliConfig.accessKeySecret)
                .setRegionId(aliConfig.getRegionId());
        return new Client(config);
    }

}
