package com.guo.gateway.cache;

import com.guo.common.constant.CacheConstants;
import com.guo.gateway.cache.abs.CacheAbs;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author gxb
 * @description 网关负载节点缓存
 * @date 2024-04-18 11:59
 */
@Component
public class LoadNodeCache extends CacheAbs<String> {


    @Override
    public String getPre() {
        return CacheConstants.GATEWAY_COMMON;
    }

    /**
     * 存负载集合
     * @param nodeList  节点权重集合
     */
    public void put(List<String> nodeList){
        //删除key
        redisService.deleteObject(encode(CacheConstants.GATEWAY_LOAD_NODE_KEY));
        //存入节点权重集合
        redisService.setCacheList(encode(CacheConstants.GATEWAY_LOAD_NODE_KEY),nodeList);
    }

    /**
     * 获取所有负载节点
     * @return 负载节点集合
     */
    public List<String> get(){
        return redisService.getCacheList(encode(CacheConstants.GATEWAY_LOAD_NODE_KEY));
    }

    /**
     * 通过下标获取节点
     * @param index  下标
     * @return   指定节点
     */
    public String getFindByIndex(Long index){
        if (index == null || index > 100){
            throw new RuntimeException("下标违法：【0 - 100】");
        }
        return redisService.getCacheListValue(encode(CacheConstants.GATEWAY_LOAD_NODE_KEY),index);
    }
}
