package com.guo.gateway.cache;

import com.guo.common.constant.CacheConstants;
import com.guo.gateway.cache.abs.CacheAbs;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @author gxb
 * @description 缓存达到缩容的节点ID
 * @date 2024-04-18 21:17
 */
@Component
public class NodeReduced extends CacheAbs<String> {



    @Override
    public String getPre() {
        return "reduced:cache:";
    }

    /**
     * 存达到缩容条件的节点ID
     * @param nodeId 节点ID
     */
    public void put(String nodeId){
        Set<String> thresholeSet = new HashSet<>();
        thresholeSet.add(nodeId);
        redisService.setCacheSetEndTime(encode(nodeId),thresholeSet, TimeUnit.MINUTES, CacheConstants.EXPIRATIOTIME);
    }

    /**
     * 判断是否被记录
     * @param nodeId 节点ID
     * @return 返回true/false
     */
    public boolean isWhether(String nodeId){
        return redisService.hasKey(encode(nodeId));
    }

    /**
     * 删除缩容条件节点ID
     */
    public void remove(String nodeId){
        redisService.deleteObject(encode(nodeId));
    }

    /**
     * 获取缩容缓存内的节点ID
     * @return
     */
    public List<String> get(String nodeId){
        return redisService.getCacheSets(encode(nodeId));
    }

    /**
     * 查询剩余时间
     * @param nodeId  节点ID
     * @return  返回剩余时间    秒
     */
    public Long remainingTime(String nodeId){
        return redisService.getExpire(encode(nodeId));
    }
}
