package com.guo.gateway.cache;

import com.alibaba.fastjson2.JSONException;
import com.alibaba.fastjson2.JSONObject;
import com.guo.common.constant.CacheConstants;
import com.guo.gateway.cache.abs.CacheAbs;
import com.guo.gateway.model.NodeJoin;
import com.guo.gateway.model.WorkGatewayNode;
import org.springframework.data.redis.core.DefaultTypedTuple;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author gxb
 * @description 网关节点连接数
 * @date 2024-04-18 14:27
 */
@Component
public class NodeScoreCache extends CacheAbs<String> {

    @Override
    public String getPre() {
        return CacheConstants.GATEWAY_COMMON;
    }

    /**
     * 获取连接数
     * @return
     */
    public List<WorkGatewayNode> getNodeScore() {
        Set<ZSetOperations.TypedTuple<Object>> range = redisService
                .redisTemplate.opsForZSet()
                .rangeWithScores(encode(CacheConstants.GATEWAY_NODE_SCORE_CACHE), 0, -1);

        if (range == null || range.isEmpty()) {
            return Collections.emptyList();
        }

        return range.stream()
                .map(tuple -> {

                    try {
                        JSONObject jsonObject = (JSONObject) tuple.getValue();

                        // 从 JSONObject 中获取字段值
                        String nodeId = jsonObject.getString("nodeId");
                        long linkingNumber = jsonObject.getLongValue("linkingNumber");

                        NodeJoin nodeJoin = new NodeJoin();
                        nodeJoin.setNodeId(nodeId);
                        nodeJoin.setLinkingNumber(linkingNumber);

                       // double score = tuple.getScore(); // 获取分数
                        return WorkGatewayNode.builder()
                                .nodeId(nodeId)
                                .weight(Integer.valueOf(String.valueOf(nodeJoin.getLinkingNumber()))) // 假设分数是double类型，这里将其转换为int类型
                                .build();
                    } catch (JSONException e) {
                        // JSON 解析异常处理
                        e.printStackTrace();
                        return null;
                    }
                })
                .filter(Objects::nonNull) // 过滤掉解析失败的节点
                .toList();
    }



    public Long getNodeNowNum(){
        List<WorkGatewayNode> workGatewayNodes = getNodeScore();
        // 直接将 double 类型结果转换为 Long 类型
        Long vehicleOnlineNowNum = (long) workGatewayNodes.stream()
                .mapToDouble(WorkGatewayNode::getWeight)
                .sum();
        return vehicleOnlineNowNum;
    }





    /**
     * 存入节点ID及连接数
     * @param nodeJoin
     */
    public void save(NodeJoin nodeJoin) {
        ZSetOperations.TypedTuple<Object> tuple = new DefaultTypedTuple<>(nodeJoin, 0.0); // 使用默认的分数为0.0
        redisService.redisTemplate.opsForZSet().add(encode(CacheConstants.GATEWAY_NODE_SCORE_CACHE), Collections.singleton(tuple));
    }



    /**
     * 获取连接数信息
     * @return
     */
    public List<String> get(){
        return redisService.getCacheObject(encode(CacheConstants.GATEWAY_NODE_SCORE_CACHE));
    }



}
