package com.guo.gateway.cache;

import com.alibaba.fastjson2.JSON;
import com.guo.common.constant.CacheConstants;
import com.guo.gateway.cache.abs.CacheAbs;
import com.guo.gateway.model.NodeVehicle;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author gxb
 * @description 网关车辆对应网关节点
 * @date 2024-04-18 14:28
 */
@Component
public class VehicleLineNodeCache extends CacheAbs<String> {


    @Override
    public String getPre() {
        return CacheConstants.GATEWAY_VEHICLE_LINE;
    }

    /**
     *存入车辆VIN 和 节点ID
     * @param nodeVehicle  车辆VIN  网关节点ID
     */
    public void save(NodeVehicle nodeVehicle){
        String key = encode(nodeVehicle.getNodeId());
        String vehicleVin = nodeVehicle.getVehicleVin();
        // Collections.singleton(vehicleVin) 来创建一个只包含一个元素 vehicleVin 的集合
        redisService.setCacheSet(key, Collections.singleton(vehicleVin));
    }

    /**
     * 获取车辆VIN信息
     * @param nodeId 网关节点ID
     * @return 返回车辆VIN，Set集合
     */

    public Set<String> get(String nodeId){
        return redisService.getCacheSet(encode(nodeId));
    }

    /**
     * 删除车辆网关连接信息
     * @param nodeId 网关节点ID
     */
    public void remove(String nodeId){
        redisService.deleteObject(encode(nodeId));
    }
}
