package com.guo.task.Contraction;

import com.guo.aly.ALYunEcsService;
import com.guo.common.constant.LoadConstants;
import com.guo.common.model.TotalNumber;
import com.guo.gateway.cache.NodeReduced;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * @author gxb
 * @description 扩缩容
 * @date 2024-04-18 19:52
 */
@AllArgsConstructor
@Component
@Log4j2
public class ContractionVolume {

    @Autowired
    private ALYunEcsService alYunEcsService;

    /**
     * 缩容缓存
     */
    private final NodeReduced nodeReduced;

    /**
     * 缩容
     * @param nodeId
     */
    public void reduction(String nodeId){
        //非空判断
        if (null != nodeId){
            //判断是否被标记过
            if (nodeReduced.isWhether(nodeId)){
                //查询剩余过期时间  秒
                Long expire = nodeReduced.remainingTime(nodeId);
                //获取 5分钟的秒值
                long fiveMinutesSeconds = TimeUnit.MINUTES.toSeconds(5);
                //打印
                log.info("空闲节点 ：" + nodeId + "的剩余时间：" + expire + "/秒");

                //比较 剩余时间 小于5分钟
                if (expire < fiveMinutesSeconds){
                    //数据迁移  释放节点

                }
            }
            //记录
            nodeReduced.put(nodeId);
        }
    }


    /**
     * 扩容
     * @param totalNumber
     */
    public void contractionVolume(TotalNumber totalNumber){
        //特殊情况   无节点
        if (totalNumber.getNodeNumber() == 0L){
            // 无   则创建两台实力
            log.error("当前未存在节点信息");
            try {
                //创建实例方法 【2台】
                 //alYunEcsService.createAnServer(LoadConstants.IS_NULL);
            } catch (Exception e) {
                log.error("扩容失败！！！！！");
                e.printStackTrace();
            }
            //返回
            return;
        }
        //调用计算
        Long value = this.percentage(totalNumber);
        //判断达到60%
        if (value >= LoadConstants.INTERMEDIATE && value < LoadConstants.MAXIMUM){
            //当节点负载达到 60%时，调用扩容一台方法
            log.info("Node 节点负载达到 ：" + value + "%，达到扩容一台的条件☑");
            try {
                //创建实例方法 【1台】
                //alYunEcsService.createAnServer(null);
            } catch (Exception e) {
                log.error("扩容失败！！！！！");
                e.printStackTrace();
            }
        }
        //判断达到80%
        if (value >= LoadConstants.MAXIMUM){
            //当节点负载达到 80%时，调用扩容一台方法
            log.info("Node 节点负载达到 ：" + value + "%，达到扩容两台的条件☑");
            try {
                //创建实例方法 【2台】
               // alYunEcsService.createAnServer(LoadConstants.IS_NULL);
            } catch (Exception e) {
                log.error("扩容失败！！！！！");
                e.printStackTrace();
            }
        }

    }

    /**
     * 计算百分比方法
     */
    private Long percentage(TotalNumber totalNumber){
        //获取节点数量
        Long nodeNumber = totalNumber.getNodeNumber();
        //根据nodeNumber去获取最大节点数  默认 100
        Long sumNodeNumber = nodeNumber * LoadConstants.MAX_NUMBER;
        //获取节点连接总数
        Long connectionTotal = totalNumber.getConnectionTotal();
        //计算空余连接数
//        Long vacantNumber = sumNodeNumber - connectionTotal;
        //计算当前负载情况
        double loadPercentage  = (double)connectionTotal / sumNodeNumber;
        //进行四舍五入取整
        long roundLoadPercentage = Math.round(loadPercentage) * LoadConstants.BE_COMMON;
        //返回百分比
        return roundLoadPercentage;
    }
}
